#!/usr/bin/perl

#
# Author: papanda@papa.to
# License: 3-clause BSD license
#

package localserver;

use strict;

sub localserver::seed {
    my ($ifname) = @_;
    my $seed = 0;
    return if (!$ifname);
    if ($ifname =~ m!^eth!) {
	use IO::Handle;
	use IO::Interface::Simple;
	
	my $ip = IO::Interface::Simple->new($ifname)->address;
	if ($ip =~ m!([\d]+)\.([\d]+)\.([\d]+)\.([\d]+)!) {
	    $seed = $4;
	} 
    }  
    return $seed;
}

sub localserver::localip {
    my ($ifname) = @_;
    return '0.0.0.0' if (!$ifname || $ifname !~ m!^eth!);
    use IO::Handle;
    use IO::Interface::Simple;
    my $ip = IO::Interface::Simple->new($ifname)->address;
    $ip = '127.0.0.1' if ($ip !~ m!([\d]+)\.!);
    return $ip;
}

sub localserver::replace {
	my ($url, $ifname) = @_;
	return if (!$ifname);
	if ($ifname =~ m!^eth!) { # Linux only
	    use IO::Handle;
	    use IO::Interface::Simple;

	    my $ip = IO::Interface::Simple->new($ifname)->address;
	    if ($ip !~ m!([\d]+)\.!) {
                $ip = '127.0.0.1';
            } 
	    $$url =~ s!LOCALSERVER!$ip!io;
	} else {
	    print STDERR "ERROR: illegal ifname: $ifname\n";
	    $$url =~ s!LOCALSERVER!$ifname!io;
	}
}

sub localserver::sendbroadcast {
    my ($ifname,$payload) = @_;
    return -1 if (!$ifname);
    if ($ifname =~ m!^eth!) {
	use IO::Handle;
	use IO::Interface::Simple;
	
	my $ip = IO::Interface::Simple->new($ifname)->address;
	if ($ip =~ m!^([\d]+)\.!) {
	    use IO::Socket::INET;
	    
	    my $broad = $ip;
	    $broad =~ s!([\d]+)$!255!;

	    my $c = IO::Socket::INET->new(
		PeerAddr  => $broad,
		PeerPort  => 10391,
		Proto     => "udp",
		Broadcast => 1
		);
	    $c->send($payload);
	    $c->close;
	    return 0;

	} else {
	    return -2;
	}
    }    
    return -3;
}

sub localserver::macaddress {
    my ($ifname) = @_;

    my $fname = "/sys/class/net/" . $ifname . "/address";
    if (-f $fname) {
	if (open(IN, "$fname")) {
	    my $macaddr = <IN>;
	    chomp $macaddr;
	    if ($macaddr =~ m!^[0-9a-fA-F:]+$!) {
		return $macaddr;
	    }
	}
    }
    use IO::Handle;
    use IO::Interface::Simple;
    my $if = IO::Interface::Simple->new($ifname);
    if ($if) {
	my $macaddr = $if->hwaddr;
	if ($macaddr =~ m!^[0-9a-fA-F:]+$!) {
	    return $macaddr;
	}
    }
    return 'ff:ff:ff:ff:ff:ff';
}

1;
